#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <Elementary.h>
#ifndef ELM_LIB_QUICKLAUNCH

typedef struct _App App;

struct _App
{
   Evas_Object *win;
   Evas_Object *bg;
   Evas_Object *browser;
   Evas_Object *url;
   Evas_Object *rotx, *roty, *rotz;
   Evas_Object *rot_enable;
   Ecore_Timer *resize_apply;
};

static int _log_domain = -1;
#define CRITICAL(...) EINA_LOG_DOM_CRIT(_log_domain, __VA_ARGS__)
#define ERR(...) EINA_LOG_DOM_ERR(_log_domain, __VA_ARGS__)
#define WRN(...) EINA_LOG_DOM_WARN(_log_domain, __VA_ARGS__)
#define INF(...) EINA_LOG_DOM_INFO(_log_domain, __VA_ARGS__)
#define DBG(...) EINA_LOG_DOM_DBG(_log_domain, __VA_ARGS__)

static void
on_win_del_req(void *data, Evas_Object *obj, void *event_info)
{
   elm_exit();
}

static void
on_slider_changed(void *data, Evas_Object *o, void *event_info)
{
   App *app = data;
   double dx = elm_slider_value_get(app->rotx);
   double dy = elm_slider_value_get(app->roty);
   double dz = elm_slider_value_get(app->rotz);
   Evas_Coord x, y, w, h;
   Evas_Map *map = evas_map_new(4);
   Eina_Bool enable = elm_check_state_get(app->rot_enable);

   if (!enable) {
      evas_object_map_set(app->browser, NULL);
      return;
   }

   evas_object_geometry_get(app->browser, &x, &y, &w, &h);

   evas_map_point_coord_set(map, 0, x,     y,     0);
   evas_map_point_coord_set(map, 1, x + w, y,     0);
   evas_map_point_coord_set(map, 2, x + w, y + h, 0);
   evas_map_point_coord_set(map, 3, x,     y + h, 0);

   evas_map_point_image_uv_set(map, 0, 0, 0);
   evas_map_point_image_uv_set(map, 1, w, 0);
   evas_map_point_image_uv_set(map, 2, w, h);
   evas_map_point_image_uv_set(map, 3, 0, h);

   x += w / 2;
   y += h / 2;

   evas_map_util_3d_rotate(map, dx, dy, dz, x, y, 0);
   evas_map_util_3d_lighting(map, -1000, -1000, -1000,
                             255, 255, 255,
                             20, 20, 20);
   evas_map_util_3d_perspective(map, x, y, 0, 512);
   evas_object_map_enable_set(app->browser, 1);
   printf("%0.3f degrees\n", dx);
   evas_object_map_set(app->browser, map);

   evas_map_free(map);
}

static void
on_check_changed(void *data, Evas_Object *o, void *event_info)
{
   App *app = data;
   Eina_Bool enable = elm_check_state_get(o);
   evas_object_map_enable_set(app->browser, enable);
   on_slider_changed(app, NULL, NULL);
}

static Eina_Bool
create_main_win(App *app)
{
   Evas_Object *box, *scroller, *entry, *frame, *slider, *o;
   char buf[4096];

   app->win = elm_win_add(NULL, "test", ELM_WIN_BASIC);
   if (!app->win)
     {
	CRITICAL("Could not create window.");
	return EINA_FALSE;
     }
   elm_win_title_set(app->win, "test");
   evas_object_smart_callback_add
     (app->win, "delete-request", on_win_del_req, NULL);

   app->bg = elm_bg_add(app->win);
   if (!app->bg)
     {
	CRITICAL("Could not create background.");
	return EINA_FALSE;
     }
   evas_object_size_hint_weight_set
     (app->bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   elm_win_resize_object_add(app->win, app->bg);
   evas_object_show(app->bg);

   o = elm_layout_add(app->win);
   snprintf(buf, sizeof(buf), "test2.edj");
   elm_layout_file_set(o, buf, "layout");
   evas_object_size_hint_weight_set(o, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   evas_object_size_hint_align_set(o, EVAS_HINT_FILL, EVAS_HINT_FILL);
   elm_win_resize_object_add(app->win, o);
   evas_object_show(o);
   
   box = elm_box_add(app->win);
   evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   elm_win_resize_object_add(app->win, box);
   evas_object_show(box);
/*
   scroller = elm_scroller_add(app->win);
   elm_scroller_content_min_limit(scroller, 0, 1);
   elm_scroller_policy_set
     (scroller, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_OFF);
   elm_scroller_bounce_set(scroller, 0, 0);
   evas_object_size_hint_weight_set(scroller, EVAS_HINT_EXPAND, 0.0);
   evas_object_size_hint_align_set(scroller, EVAS_HINT_FILL, EVAS_HINT_FILL);
   evas_object_show(scroller);
   elm_box_pack_end(box, scroller);

   entry = elm_entry_add(app->win);
   elm_entry_single_line_set(entry, 1);
   evas_object_size_hint_weight_set(entry, EVAS_HINT_EXPAND, 0.0);
   evas_object_size_hint_align_set(entry, EVAS_HINT_FILL, 0.5);
   evas_object_show(entry);
   elm_scroller_content_set(scroller, entry);
   app->url = entry;
 */
#if 0   
   app->browser = evas_object_rectangle_add(evas_object_evas_get(app->bg));
   evas_object_color_set(app->browser, 255, 255, 0, 255);
   evas_object_size_hint_weight_set
     (app->browser, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   evas_object_size_hint_align_set
     (app->browser, EVAS_HINT_FILL, EVAS_HINT_FILL);
#else
   app->browser = elm_table_add(app->win);
   evas_object_size_hint_weight_set
     (app->browser, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   evas_object_size_hint_align_set
     (app->browser, EVAS_HINT_FILL, EVAS_HINT_FILL);

   o = elm_layout_add(app->win);
   snprintf(buf, sizeof(buf), "test.edj");
   elm_layout_file_set(o, buf, "layout");
   evas_object_size_hint_weight_set(o, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
   evas_object_size_hint_align_set(o, EVAS_HINT_FILL, EVAS_HINT_FILL);
   elm_table_pack(app->browser, o, 0, 0, 3, 3);
   evas_object_show(o);
   
   o = elm_slider_add(app->win);
   elm_slider_label_set(o, "Slide me!");
   elm_slider_span_size_set(o, 200);
   elm_slider_unit_format_set(o, "%0.0f");
   elm_slider_horizontal_set(o, 1);
   elm_slider_min_max_set(o, 0, 100);
   elm_table_pack(app->browser, o, 1, 0, 1, 1);
   evas_object_show(o);
   
   o = elm_slider_add(app->win);
   elm_slider_label_set(o, "Slide me!");
   elm_slider_span_size_set(o, 200);
   elm_slider_unit_format_set(o, "%0.0f");
   elm_slider_horizontal_set(o, 1);
   elm_slider_min_max_set(o, 0, 100);
   elm_table_pack(app->browser, o, 1, 2, 1, 1);
   evas_object_show(o);

   o = elm_button_add(app->win);
   elm_button_label_set(o, "Button");
   elm_table_pack(app->browser, o, 1, 1, 1, 1);
   evas_object_show(o);
   
   o = elm_slider_add(app->win);
   elm_slider_label_set(o, "Slide me!");
   elm_slider_span_size_set(o, 200);
   elm_slider_unit_format_set(o, "%0.0f");
   elm_slider_horizontal_set(o, 0);
   elm_slider_min_max_set(o, 0, 100);
   elm_table_pack(app->browser, o, 0, 0, 1, 3);
   evas_object_show(o);
   
   o = elm_slider_add(app->win);
   elm_slider_label_set(o, "Slide me!");
   elm_slider_span_size_set(o, 200);
   elm_slider_unit_format_set(o, "%0.0f");
   elm_slider_horizontal_set(o, 0);
   elm_slider_min_max_set(o, 0, 100);
   elm_table_pack(app->browser, o, 2, 0, 1, 3);
   evas_object_show(o);
   
#endif   
   evas_object_show(app->browser);
   elm_box_pack_end(box, app->browser);

   frame = elm_frame_add(app->win);
   elm_frame_label_set(frame, "Rotation");
   evas_object_size_hint_min_set(frame, 200, 200);
   evas_object_size_hint_align_set(frame, EVAS_HINT_FILL, EVAS_HINT_FILL);
   evas_object_show(frame);
   elm_box_pack_end(box, frame);

   box = elm_box_add(app->win);
   evas_object_size_hint_align_set(box, EVAS_HINT_FILL, EVAS_HINT_FILL);
   elm_frame_content_set(frame, box);
   evas_object_show(box);

   slider = elm_slider_add(app->win);
   elm_slider_label_set(slider, "degrees:");
   elm_slider_span_size_set(slider, 40);
   elm_slider_unit_format_set(slider, "%0.0f");
   elm_slider_horizontal_set(slider, 1);
   elm_slider_min_max_set(slider, 0.0, 360.0);
   evas_object_smart_callback_add
     (slider, "changed", on_slider_changed, app);
   evas_object_size_hint_align_set(slider, EVAS_HINT_FILL, EVAS_HINT_FILL);
   evas_object_show(slider);
   elm_box_pack_end(box, slider);
   app->rotx = slider;

   slider = elm_slider_add(app->win);
   elm_slider_label_set(slider, "degrees:");
   elm_slider_span_size_set(slider, 40);
   elm_slider_unit_format_set(slider, "%0.0f");
   elm_slider_horizontal_set(slider, 1);
   elm_slider_min_max_set(slider, 0.0, 360.0);
   evas_object_smart_callback_add
     (slider, "changed", on_slider_changed, app);
   evas_object_size_hint_align_set(slider, EVAS_HINT_FILL, EVAS_HINT_FILL);
   evas_object_show(slider);
   elm_box_pack_end(box, slider);
   app->roty = slider;

   slider = elm_slider_add(app->win);
   elm_slider_label_set(slider, "degrees:");
   elm_slider_span_size_set(slider, 40);
   elm_slider_unit_format_set(slider, "%0.0f");
   elm_slider_horizontal_set(slider, 1);
   elm_slider_min_max_set(slider, 0.0, 360.0);
   evas_object_smart_callback_add
     (slider, "changed", on_slider_changed, app);
   evas_object_size_hint_align_set(slider, EVAS_HINT_FILL, EVAS_HINT_FILL);
   evas_object_show(slider);
   elm_box_pack_end(box, slider);
   app->rotz = slider;

   app->rot_enable = elm_check_add(app->win);
   elm_check_label_set(app->rot_enable, "enable rotation.");
   elm_check_state_set(app->rot_enable, 0);
   evas_object_smart_callback_add
     (app->rot_enable, "changed", on_check_changed, app);
   evas_object_show(app->rot_enable);
   elm_box_pack_end(box, app->rot_enable);

   evas_object_resize(app->win, 640, 640);
   evas_object_show(app->win);

   return EINA_TRUE;
}

EAPI int
elm_main(int argc, char **argv)
{
   App app;
   int r = 0;

#if ENABLE_NLS
   setlocale(LC_ALL, "");
   bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
   bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
   textdomain(GETTEXT_PACKAGE);
#endif

/*    ewk_init(); */

   _log_domain = eina_log_domain_register("test", NULL);
   if (_log_domain < 0)
     {
	EINA_LOG_CRIT("could not create log domain 'test'.");
	return -1;
     }

   memset(&app, 0, sizeof(app));
   if (!create_main_win(&app))
     {
	r = -1;
	goto end;
     }


   elm_run();
 end:
   eina_log_domain_unregister(_log_domain);
   _log_domain = -1;
/*    ewk_shutdown(); */
   elm_shutdown();
   return r;
}
#endif
ELM_MAIN()
